package com.supermap.dataImage;

import android.app.Activity;
import android.app.AlertDialog;
import android.content.DialogInterface;
import android.content.DialogInterface.OnMultiChoiceClickListener;
import android.content.pm.ActivityInfo;
import android.os.Bundle;
import android.view.View;
import android.view.WindowManager;
import android.view.View.OnClickListener;
import android.widget.Button;
import android.widget.ListView;
import android.widget.Toast;

import com.supermap.analyst.spatialanalyst.RasterClip;
import com.supermap.analyst.spatialanalyst.RasterClipFileType;
import com.supermap.data.DataConversion;
import com.supermap.data.Dataset;
import com.supermap.data.DatasetImage;
import com.supermap.data.Datasets;
import com.supermap.data.Datasource;
import com.supermap.data.Environment;
import com.supermap.data.GeoLine;
import com.supermap.data.GeoRegion;
import com.supermap.data.Point2D;
import com.supermap.data.Point2Ds;
import com.supermap.data.Workspace;
import com.supermap.data.WorkspaceConnectionInfo;
import com.supermap.data.WorkspaceType;
import com.supermap.mapping.ImageStretchOption;
import com.supermap.mapping.ImageStretchType;
import com.supermap.mapping.Layer;
import com.supermap.mapping.LayerSettingImage;
import com.supermap.mapping.MapControl;
import com.supermap.mapping.MapView;
/**
 * <p>
 * Title: Remote sensing image data processing
 * </p>
 * 
 * <p>
 * Description:
 * ============================================================================>
 * ------------------------------Statement----------------------------
 * This is for SuperMap iMobile for Android Sample Code 
 * Copyright: SuperMap Software Co., Ltd.
 * ----------------------------------------------------------------
 * ----------------------------SuperMap iMobile
 * Demo---------------------------
 * 
 *  1. Introduction: Operate the remote sensing image data.
 * 2. Demo data: 
 * Data directory: "SampleData/DataImage/tifData" 
 * Map data: "World.smwu", "World.udb","World.udd" 
 * License: "/SuperMap/License/" 
 * 3. Members: 
 * DataConversion.importTIF(); Method 
 * DataConversion.exportTIF(); Method
 * RasterClip.clip(); Method 
 * RasterClip.split(); Method
 * LayerSettingImage.setDisplayBandIndexes();Method
 * LayerSettingImage.getDisplayBandIndexes();Method
 * LayerSettingImage.setImageStretchOption();Method
 * ImageStretchOption.setStretchType(); Method
 * 
 * 4. Function display:
 * (1)Remote sensing image data import
 * (2)Remote sensing image data export
 * (3)Band display
 * (4)Stretch Type
 * (5)Clipping data
 * (6)Split the data
 * (7)The splitting data is the image data
 * --------------------------------------------
 * ----------------------------------
 * ============================================================================>
 * </p>
 * 
 * <p>
 * Company: SuperMap Software Co., Ltd.
 * </p>
 * 
 */
public class MainActivity extends Activity implements OnClickListener {
	private String sdCard = android.os.Environment
			.getExternalStorageDirectory().getAbsolutePath();
	private Workspace mWorkspace;
	private MapView mMapView;
	private MapControl mMapControl;
	private Button btnExport;
	private Button btnImport;
	private Button btnCutting;
	private Button btnCutImage;
	private Button btnCutDataset;
	private Button btnband;
	private Button btnstretching;
	private String tifImportPath = sdCard
			+ "/SampleData/DataImage/tifData/WorldEarth.tif";
	private String tifExportPath = sdCard
			+ "/SampleData/DataImage/tifData/tifTest.tif";
	private String cutImagePath = sdCard
			+ "/SampleData/DataImage/tifData/cutImageTest.tif";
	private Datasource udbDatasource;
	private Dataset dataset;
	private Layer mLayer;
	private boolean resultBoolean;
	private LayerSettingImage layerSettingImage;
	private String[] StringArrays;
	private int[] intArrays = new int[] {0, 1, 2};
	private Point2D point1;
	private Point2D point2;
	private Point2D point3;
	private Point2D point4;
	private Point2D point5;
	private Point2D point6;
	private Point2D point7;
	private double pointWidth;
	private double poinTop;
	private double poinBottom;
	private double poinLeft;
	private Dataset resultdataset;
	private Point2D[] point2d;
	private Point2Ds point2Ds;
	private GeoRegion region;
	private GeoLine Line;
	private boolean[] areaStates;
	private ListView listview;


	@Override
	protected void onCreate(Bundle savedInstanceState) {
		super.onCreate(savedInstanceState);
		Environment.initialization(this);
		Environment.setOpenGLMode(false);
		Environment.setLicensePath(sdCard + "/SuperMap/license");
		//Set up the landscape
		if( getRequestedOrientation() != ActivityInfo.SCREEN_ORIENTATION_LANDSCAPE ){
			setRequestedOrientation(ActivityInfo.SCREEN_ORIENTATION_LANDSCAPE);
		}
		setContentView(R.layout.activity_main);

		initView();		//Initialize the view control
		openMap();		//Open the map
	}

	/**
	 * Open the map
	 */
	private void openMap() {
		mWorkspace = new Workspace();
		WorkspaceConnectionInfo info = new WorkspaceConnectionInfo();
		info.setServer(sdCard + "/SampleData/DataImage/tifData/TIF.smwu");
		info.setType(WorkspaceType.SMWU);
		mWorkspace.open(info);
		// The map display controls are associated to the workspace
		mMapView = (MapView) findViewById(R.id.mapView);
		mMapControl = mMapView.getMapControl();
		mMapControl.getMap().setWorkspace(mWorkspace);
		mMapControl.getMap().open("World");
		mMapControl.getMap().refresh();
		
		udbDatasource = mMapControl.getMap().getWorkspace().getDatasources()
				.get("tif");
		
		//The current editable layer
		mLayer = mMapControl.getMap().getLayers().get(0);
		mLayer.setVisible(true);
		mLayer.setEditable(true);
	}

	/**
	 * Initial control
	 */
	private void initView() {

		btnCutting = (Button) findViewById(R.id.btn_cutting);
		btnCutImage = (Button) findViewById(R.id.btn_Cut_Image);
		btnCutDataset = (Button) findViewById(R.id.btn_Cut_Dataset);

		btnCutting.setOnClickListener(this);
		btnCutImage.setOnClickListener(this);
		btnCutDataset.setOnClickListener(this);

		findViewById(R.id.btnStretch2).setOnClickListener(this);
		findViewById(R.id.btnStretch3).setOnClickListener(this);
		findViewById(R.id.btnStretch4).setOnClickListener(this);
		findViewById(R.id.btnStretch5).setOnClickListener(this);
		findViewById(R.id.btnStretch6).setOnClickListener(this);
		findViewById(R.id.btnStretch7).setOnClickListener(this);
		
		findViewById(R.id.btnClip2).setOnClickListener(this);
		findViewById(R.id.btnClip3).setOnClickListener(this);
		findViewById(R.id.btnClip4).setOnClickListener(this);
		
		findViewById(R.id.btnControlDisplay2).setOnClickListener(this);
		findViewById(R.id.btnControlDisplay3).setOnClickListener(this);
		findViewById(R.id.btnControlDisplay4).setOnClickListener(this);

	}

	/**
	 * Button 
	 */
	@Override
	public void onClick(View v) {
		switch (v.getId()) {

		case R.id.btn_cutting:// Split the dataset
			showButton(2);
			break;
		case R.id.btnControlDisplay2:
			bandStyle(0);
			break;
		case R.id.btnControlDisplay3:
			bandStyle(1);
			break;
		case R.id.btnControlDisplay4:
			bandStyle(2);
			break;
		case R.id.btn_Cut_Dataset:// Clip to the dataset
			showButton(1);
			break;
		case R.id.btnClip2://Clip to dataset
			cutDataset();
			break;
		case R.id.btnClip3://Clip to file
			cutImage();
			break;
		case R.id.btnClip4://Image clipping
			cuttingDataset();
			break;
		case R.id.btn_Cut_Image:// Clipping to image data
			showButton(0);
			break;

		case R.id.btnStretch2://Gaussian stretch
			setGaussian();
			break;
		case R.id.btnStretch3://The extreme value streching
			setMinimummaximum();
			break;
		case R.id.btnStretch4://The standard deviation streching
			setStandarddeviation();
			break;

		case R.id.btnStretch5://Histogram matching
			setHistogramspecification();
			break;
		case R.id.btnStretch6://Histogram equalization
			setHistogramequalization();
		case R.id.btnStretch7://No Stretch
			setNone();
			break;
		default:
			break;
		}
	}
	/**
	 * Determine whether the dataset is null
	 */
	private boolean notNull(){
		dataset = udbDatasource.getDatasets().get("WorldEarth");
		if(dataset== null){
			Toast.makeText(getApplicationContext(), "Please import the data", Toast.LENGTH_SHORT).show();
			return false;
		}
		return true;
	}
	/**
	 * Streching type
	 */
	private void stretchingType() {
		if(notNull()!=true){
			return;
		}
		AlertDialog.Builder builder = new AlertDialog.Builder(MainActivity.this);
		builder.setIcon(R.drawable.ic_launcher);
		builder.setTitle("Select the streching type");
		// The display data
		final String[] cities = { "Gaussian", "Histogram equalization", "Histogram matching", "Extreme value", "No Stretch",
				"Standard deviation" };
		// Set a drop-down list options
		builder.setItems(cities, new DialogInterface.OnClickListener() {

			@Override
			public void onClick(DialogInterface dialog, int which) {
				switch (which) {
				case 0:
	            	//Gaussian stretch
					setGaussian();
					break;
				case 1:
	            	//Histogram equalization
					setHistogramequalization();
					break;

				case 2:
	             	//Histogram matching
					setHistogramspecification();
					break;

				case 3:
					//The extreme value streching
					setMinimummaximum();
					break;

				case 4:
					//No Stretch
					setNone();
					break;

				case 5:
					//The standard deviation streching
					setStandarddeviation();
					break;
				default:
					break;
				}
			}
		}).setNegativeButton("Cancel", null).create();
		builder.show();
	}

	
	/**
	 * Band display style
	 */
	private void bandStyle(final int colorIndex) {

		listview = new ListView(getApplicationContext());
		StringArrays = new String[] { "Band1", "Band2", "Band3" };
		areaStates = new boolean[] { false, false, false };
		AlertDialog builder = new AlertDialog.Builder(this)
				.setTitle("Select the display band")
				.setMultiChoiceItems(StringArrays, areaStates,
						new OnMultiChoiceClickListener() {

							@Override
							public void onClick(DialogInterface dialog,
									int which, boolean isChecked) {
							}
						})
				.setPositiveButton("OK", new DialogInterface.OnClickListener() {

					public void onClick(DialogInterface dialog, int id) {
						for (int i = 0; i < intArrays.length; i++) {
							if (listview.getCheckedItemPositions().get(i)) {
								intArrays[colorIndex] = i;
							}
						}
						layerSettingImage = (LayerSettingImage) mLayer
								.getAdditionalSetting();
						layerSettingImage.setDisplayBandIndexes(intArrays);
						
						mMapControl.getMap().refresh();
						Toast.makeText(getApplicationContext(), "Set display band successfully",
								Toast.LENGTH_SHORT).show();
					}
				}).setNegativeButton("Cancel", null).create();
		listview = builder.getListView();
		builder.show();
	}

	/**
	 * Split the dataset
	 */
	private void cuttingDataset() {
		cutLine();
		Datasets datasets = udbDatasource.getDatasets();
		dataset = mLayer.getDataset();
		if(dataset.getName().equals("LeftDataName"))
			return; // Splitted
		
		if(datasets.contains("LeftDataName")){
			datasets.delete("LeftDataName");
		}
		if (datasets.contains("RightDataName")) {
			datasets.delete("RightDataName");
		}
		
		resultBoolean = RasterClip.split(dataset, Line, udbDatasource,
				"LeftDataName", "RightDataName");
		if (resultBoolean) {
			mMapControl.getMap().close();
			mLayer = mMapControl.getMap().getLayers().add(datasets.get("LeftDataName"), true);
			mMapControl.getMap().refresh();
			Toast.makeText(getApplicationContext(), "Successfully", Toast.LENGTH_SHORT)
					.show();
		} else {
			Toast.makeText(getApplicationContext(), "Failed", Toast.LENGTH_SHORT)
					.show();
		}
		
		mMapControl.getMap().getTrackingLayer().clear();
	}

	/**
	 * Splitting line 
	 */
	private void cutLine() {
		point6 = new Point2D();
		point7 = new Point2D();
		pointWidth = mLayer.getDataset().getBounds().getWidth() / 2;//udbDatasource.getDatasets().get("WorldEarth").getBounds().getWidth() / 2;
		poinTop = mLayer.getDataset().getBounds().getTop();//udbDatasource.getDatasets().get("WorldEarth").getBounds().getTop();
		poinBottom = mLayer.getDataset().getBounds().getBottom();//udbDatasource.getDatasets().get("WorldEarth").getBounds().getBottom();
		poinLeft = mLayer.getDataset().getBounds().getLeft();//udbDatasource.getDatasets().get("WorldEarth").getBounds().getLeft();
		point6.setX(pointWidth + poinLeft);
		point6.setY(poinTop);
		point7.setX(pointWidth + poinLeft);
		point7.setY(poinBottom);
		point2d = new Point2D[] { point6, point7 };
		point2Ds = new Point2Ds();
		point2Ds.addRange(point2d);
		Line = new GeoLine();
		Line.addPart(point2Ds);
		
		mMapControl.getMap().getTrackingLayer().add(Line, "line");
		mMapControl.getMap().refresh();
	}

	/**
	 * Split to the image data
	 */
	private void cutImage() {

		DatasetImage image = (DatasetImage)mLayer.getDataset();//(DatasetImage) udbDatasource.getDatasets().get("WorldEarth");
		regionSet();
		resultBoolean = RasterClip.clip(image, region, true, cutImagePath,
				RasterClipFileType.TIF);
		if (resultBoolean) {
			Toast.makeText(getApplicationContext(), "Clip the image dataset successfully: " + cutImagePath,
					Toast.LENGTH_SHORT).show();
		} else {
			Toast.makeText(getApplicationContext(), "Faile to clip the image dataset",
					Toast.LENGTH_SHORT).show();
		}
	}

	/**
	 * Clip to the dataset
	 */
	private void cutDataset() {

		regionSet();
		Datasets datasets = udbDatasource.getDatasets();
		dataset = mLayer.getDataset();//udbDatasource.getDatasets().get("WorldEarth");
		if(dataset.getName().equals("cutDataset"))
			return; // clipped
		
		if(datasets.contains("cutDataset")){
			datasets.delete("cutDataset");
		}

		resultdataset = RasterClip.clip(dataset, region, true, true,
				udbDatasource, "cutDataset");
		if (resultdataset != null) {
			mMapControl.getMap().getLayers().remove("WorldEarth@WorldEarth");
			dataset = udbDatasource.getDatasets().get("cutDataset");
			//addMap();
			mLayer = mMapControl.getMap().getLayers().add(dataset, true);
			mLayer.setVisible(true);
			mLayer.setEditable(true);
			mMapControl.getMap().refresh();
			
			Toast.makeText(getApplicationContext(), "Clip to the dataset successfully",
					Toast.LENGTH_SHORT).show();
		} else {
			Toast.makeText(getApplicationContext(), "Failed to clip to the dataset",
					Toast.LENGTH_SHORT).show();
		}
	}

	/**
	 * Clip the rectangular area
	 */
	private void regionSet() {
		point1 = new Point2D();
		point2 = new Point2D();
		point3 = new Point2D();
		point4 = new Point2D();
		point5 = new Point2D();
		point1.setX(20.34);
		point1.setY(20.34);
		point2.setX(100.34);
		point2.setY(20.34);
		point3.setX(100.34);
		point3.setY(100.34);
		point4.setX(20.34);
		point4.setY(100.34);
		point5.setX(20.34);
		point5.setY(20.34);
		point2d = new Point2D[] { point1, point2, point3, point4, point5 };
		point2Ds = new Point2Ds();
		point2Ds.addRange(point2d);
		region = new GeoRegion();
		region.addPart(point2Ds);
	}

	/**
	 * Export the datasource
	 */
	private void exportDataset() {
		if(!notNull()){
			return;
		}
		try {
			resultBoolean = DataConversion.exportTIF(tifExportPath, dataset);
			if (resultBoolean == true) {
				dataset = udbDatasource.getDatasets().get("WorldEarth");
				addMap();
				Toast.makeText(getApplicationContext(), "Image data is exported successfully",
						Toast.LENGTH_SHORT).show();
			} else {
				Toast.makeText(getApplicationContext(), "Failed to export the image data",
						Toast.LENGTH_SHORT).show();
			}
		} catch (Exception e) {
			e.printStackTrace();
		}

	}

	/**
	 * Import the image data
	 */
	private void importImageData() {
		try {
			Datasets datasets =udbDatasource.getDatasets();
			if(datasets.contains("WorldEarth")){
				datasets.delete("WorldEarth");
			}
			
			resultBoolean = DataConversion.importTIF(tifImportPath,
					udbDatasource);
			if (resultBoolean == true) {
				dataset = udbDatasource.getDatasets().get("WorldEarth");
				addMap();
				Toast.makeText(getApplicationContext(), "Image data is imported successfully",
						Toast.LENGTH_SHORT).show();
			} else {
				Toast.makeText(getApplicationContext(), "Failed to import the image data",
						Toast.LENGTH_SHORT).show();
			}
		} catch (Exception e) {
			e.printStackTrace();
		}

	}

	/**
	 * Add the dataset to the map layer
	 */
	private void addMap() {
		mLayer = mMapControl.getMap().getLayers().add(dataset, true);
		mLayer.setVisible(true);
		mLayer.setEditable(true);
		mMapControl.getMap().refresh();
	}

	/**
	 * Set the gaussian streching
	 */
	public void setGaussian() {
		ImageStretchOption imageStretchOption = new ImageStretchOption();
		// Get the layer display mode
		layerSettingImage = (LayerSettingImage) mLayer.getAdditionalSetting();
		// Set the streching style
		imageStretchOption.setStretchType(ImageStretchType.GAUSSIAN);
		// Set the layer streshing style
		layerSettingImage.setImageStretchOption(imageStretchOption);
		// Refresh the layer;
		mMapControl.getMap().refresh();
		Toast.makeText(getApplicationContext(), "Set the streching style successfully", Toast.LENGTH_SHORT)
				.show();
	}

	/**
	 * Set the histogram equalization
	 */
	public void setHistogramequalization() {
		ImageStretchOption imageStretchOption = new ImageStretchOption();
		layerSettingImage = (LayerSettingImage) mLayer.getAdditionalSetting();
		imageStretchOption
				.setStretchType(ImageStretchType.HISTOGRAMEQUALIZATION);
		layerSettingImage.setImageStretchOption(imageStretchOption);
		mMapControl.getMap().refresh();
		Toast.makeText(getApplicationContext(), "Set the streching style successfully", Toast.LENGTH_SHORT)
				.show();

	}

	/**
	 * Set the histogram matching
	 */
	public void setHistogramspecification() {
		ImageStretchOption imageStretchOption = new ImageStretchOption();
		layerSettingImage = (LayerSettingImage) mLayer.getAdditionalSetting();
		imageStretchOption
				.setStretchType(ImageStretchType.HISTOGRAMSPECIFICATION);
		layerSettingImage.setImageStretchOption(imageStretchOption);
		mMapControl.getMap().refresh();
		Toast.makeText(getApplicationContext(), "Set the streching style successfully", Toast.LENGTH_SHORT)
				.show();

	}

	/**
	 * Set the extreme value streching
	 */
	public void setMinimummaximum() {
		ImageStretchOption imageStretchOption = new ImageStretchOption();
		layerSettingImage = (LayerSettingImage) mLayer.getAdditionalSetting();
		imageStretchOption.setStretchType(ImageStretchType.MINIMUMMAXIMUM);
		layerSettingImage.setImageStretchOption(imageStretchOption);
		mMapControl.getMap().refresh();
		Toast.makeText(getApplicationContext(), "Set the streching style successfully", Toast.LENGTH_SHORT)
				.show();

	}

	/**
	 * Set the non-streching
	 */
	public void setNone() {
		ImageStretchOption imageStretchOption = new ImageStretchOption();
		layerSettingImage = (LayerSettingImage) mLayer.getAdditionalSetting();
		imageStretchOption.setStretchType(ImageStretchType.NONE);
		layerSettingImage.setImageStretchOption(imageStretchOption);
		mMapControl.getMap().refresh();
		Toast.makeText(getApplicationContext(), "Set the streching style successfully", Toast.LENGTH_SHORT)
				.show();
	}

	/**
	 * Set the standard deviation streching
	 */
	public void setStandarddeviation() {
		ImageStretchOption imageStretchOption = new ImageStretchOption();
		layerSettingImage = (LayerSettingImage) mLayer.getAdditionalSetting();
		imageStretchOption.setStretchType(ImageStretchType.STANDARDDEVIATION);
		layerSettingImage.setImageStretchOption(imageStretchOption);
		mMapControl.getMap().refresh();
		Toast.makeText(getApplicationContext(), "Set the streching style successfully", Toast.LENGTH_SHORT)
				.show();

	}

	public void showButton(int index) {
		switch (index) {
			case 0: {
				findViewById(R.id.btnStretch2).setVisibility(View.VISIBLE);
				findViewById(R.id.btnStretch3).setVisibility(View.VISIBLE);
				findViewById(R.id.btnStretch4).setVisibility(View.VISIBLE);
				findViewById(R.id.btnStretch5).setVisibility(View.VISIBLE);
				findViewById(R.id.btnStretch6).setVisibility(View.VISIBLE);
				findViewById(R.id.btnStretch7).setVisibility(View.VISIBLE);

				findViewById(R.id.btnClip2).setVisibility(View.INVISIBLE);
				findViewById(R.id.btnClip3).setVisibility(View.INVISIBLE);
				findViewById(R.id.btnClip4).setVisibility(View.INVISIBLE);

				findViewById(R.id.btnControlDisplay2).setVisibility(View.INVISIBLE);
				findViewById(R.id.btnControlDisplay3).setVisibility(View.INVISIBLE);
				findViewById(R.id.btnControlDisplay4).setVisibility(View.INVISIBLE);
			}
			break;
			case 1: {
				findViewById(R.id.btnStretch2).setVisibility(View.INVISIBLE);
				findViewById(R.id.btnStretch3).setVisibility(View.INVISIBLE);
				findViewById(R.id.btnStretch4).setVisibility(View.INVISIBLE);
				findViewById(R.id.btnStretch5).setVisibility(View.INVISIBLE);
				findViewById(R.id.btnStretch6).setVisibility(View.INVISIBLE);
				findViewById(R.id.btnStretch7).setVisibility(View.INVISIBLE);

				findViewById(R.id.btnClip2).setVisibility(View.VISIBLE);
				findViewById(R.id.btnClip3).setVisibility(View.VISIBLE);
				findViewById(R.id.btnClip4).setVisibility(View.VISIBLE);

				findViewById(R.id.btnControlDisplay2).setVisibility(View.INVISIBLE);
				findViewById(R.id.btnControlDisplay3).setVisibility(View.INVISIBLE);
				findViewById(R.id.btnControlDisplay4).setVisibility(View.INVISIBLE);
			}
			break;
			case 2: {

				findViewById(R.id.btnStretch2).setVisibility(View.INVISIBLE);
				findViewById(R.id.btnStretch3).setVisibility(View.INVISIBLE);
				findViewById(R.id.btnStretch4).setVisibility(View.INVISIBLE);
				findViewById(R.id.btnStretch5).setVisibility(View.INVISIBLE);
				findViewById(R.id.btnStretch6).setVisibility(View.INVISIBLE);
				findViewById(R.id.btnStretch7).setVisibility(View.INVISIBLE);

				findViewById(R.id.btnClip2).setVisibility(View.INVISIBLE);
				findViewById(R.id.btnClip3).setVisibility(View.INVISIBLE);
				findViewById(R.id.btnClip4).setVisibility(View.INVISIBLE);

				findViewById(R.id.btnControlDisplay2).setVisibility(View.VISIBLE);
				findViewById(R.id.btnControlDisplay3).setVisibility(View.VISIBLE);
				findViewById(R.id.btnControlDisplay4).setVisibility(View.VISIBLE);
			}
			break;
		}
		mMapControl.getMap().refresh();
	}
}


